@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2019
+/* Copyright (c) 1997-2021
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_POLYTOPE_CUBE_FACETS_H
-#define POLYMAKE_POLYTOPE_CUBE_FACETS_H
+#pragma once
 
 #include "polymake/GenericSet.h"
 
@@ -141,10 +140,10 @@ public:
    /** @param dim_arg dimension of the cube
        @param start_arg index of the first vertex
    */
-   explicit CubeFacets(int dim_arg, const E& start_arg=E(0))
+   explicit CubeFacets(Int dim_arg, const E& start_arg = zero_value<E>())
       : start(start_arg), dim(dim_arg) { }
 
-   int size() const { return 2*dim; }
+   Int size() const { return 2*dim; }
    bool empty() const { return dim==0; }
 
    iterator begin() const { return iterator(start, E(1), E(1)<<dim); }
@@ -153,7 +152,7 @@ public:
    const value_type front() const { return value_type(start, E(1), E(1)<<dim); }
 protected:
    E start;
-   int dim;
+   Int dim;
 };
 
 } }
@@ -167,7 +166,6 @@ template <typename E>
 struct check_iterator_feature<polymake::polytope::CubeFacets_iterator<E>, end_sensitive> : std::true_type { };
 }
 
-#endif // POLYMAKE_POLYTOPE_CUBE_FACETS_H
 
 // Local Variables:
 // mode:C++
