@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2015
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2021
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -14,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_POLYTOPE_CANONICALIZE_H
-#define POLYMAKE_POLYTOPE_CANONICALIZE_H
+#pragma once
 
 
 #include "polymake/client.h"
@@ -28,7 +28,9 @@ template <typename Iterator> inline
 void canonicalize_oriented(Iterator&& it)
 {
   typedef typename pm::iterator_traits<Iterator>::value_type Scalar;
-   if (!it.at_end() && !abs_equal(*it, one_value<Scalar>())) {
+  while (!it.at_end() && is_zero(*it))
+    ++it;
+  if (!it.at_end() && !abs_equal(*it, one_value<Scalar>())) {
       const Scalar leading=abs(*it);
       auto&& tail=ensure_private_mutable(std::forward<Iterator>(it));
       do *tail /= leading; while (!(++tail).at_end());
@@ -37,4 +39,3 @@ void canonicalize_oriented(Iterator&& it)
 
 } }
 
-#endif
