@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2018
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2021
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -27,15 +28,15 @@ use Term::Cap;
 my ($bold, $boldoff, $under, $underoff);
 
 sub init_termcap {
-   state $termcap= $ENV{TERM} && do {
-      my $tc=Term::Cap->Tgetent;
-      $bold=$tc->Tputs('md');
-      $boldoff=$tc->Tputs('me');
-      $under=$tc->Tputs('us');
-      $underoff=$tc->Tputs('ue');
+   state $termcap = defined($ENV{TERM}) && eval {
+      my $tc = Term::Cap->Tgetent;
+      $bold = $tc->Tputs('md');
+      $boldoff = $tc->Tputs('me');
+      $under = $tc->Tputs('us');
+      $underoff = $tc->Tputs('ue');
       close Term::Cap::DATA;
       1
-   };
+   } // (warn("Term::Cap->Tgetent failed for TERM $ENV{TERM}: $@"), 0);
 }
 
 sub Tgetent {
@@ -99,12 +100,10 @@ sub do_reconfigure {
       local if (!defined($self->compile_scope)) {
          local unshift @INC, $self;
          local scalar $self->compile_scope = new Scope();
-         local ref $self->custom->tied_vars = [];
-         $self->compile_scope->cleanup->{$self->custom} = undef;
       }
       while (($filename, $ext, $rule_key) = splice @$list, 0, 3) {
          my $rc;
-         local $extension = $ext if $ext != $extension;
+         local $Extension::loading = $ext;
          if (defined (my $code = $self->rule_code->{$rule_key})) {
             my $reconf = new Reconf($self, false);
             no strict 'refs';
@@ -115,7 +114,7 @@ sub do_reconfigure {
             &$code;
             $rc = $self->rulefiles->{$rule_key};
          } else {
-            $rc=parse_rulefile($self, $rule_key, $filename);
+            $rc = parse_rulefile($self, $rule_key, $filename);
          }
          if ($rc && defined (my $to_wake=$rules_to_wake{$self})) {
             if (defined ($to_wake=delete $to_wake->{$rule_key})) {
@@ -128,7 +127,7 @@ sub do_reconfigure {
       @{$self->rules_to_finalize} = ();
 
       for (; $new_to_wake < $#woken; $new_to_wake += 2) {
-         ($self, $list) = @woken[$new_to_wake, $new_to_wake + 1];
+         ($self, $list) = @woken[$new_to_wake, $new_to_wake+1];
          for (my $i = 0; $i < @$list; $i += 3) {
             ($filename, $ext, $rule_key) = @$list[$i..$i+2];
             delete $self->configured->{$rule_key};
@@ -141,8 +140,7 @@ sub lookup_rulefile_or_key {
    my ($self, $rulefile) = @_;
    if ($rulefile =~ s/\@(.*)$//) {
       # specified by the key including extension URI
-      local $extension = $Extension::registered_by_URI{$1};
-      if (defined $extension) {
+      if (local $Extension::loading = $Extension::registered_by_URI{$1}) {
          lookup_rulefile($self, $rulefile, 1);
       } else {
          die "unknown extension URI $1\n";
@@ -195,11 +193,10 @@ sub unconfigure {
             my $unconf = new Unconf($self);
             no strict 'refs';
             local ref *{$self->pkg."::self"} = sub { $unconf };
-            local $extension = $ext;
+            local $Extension::loading = $ext;
             &$code;
          } else {
             $self->configured->{$rule_key} = -$load_time;
-            $self->custom->handler->need_save = 1;
          }
       } elsif (defined $config_state) {
          die "rulefile $rulefile is already unconfigured\n";
@@ -213,24 +210,22 @@ sub unconfigure {
 ###############################################################################################
 # private:
 sub valid_configured_entry {
-   my ($self, $rule_key)=@_;
+   my ($self, $rule_key) = @_;
    exists $self->rulefiles->{$rule_key} or do {
       if ($rule_key =~ m{^/}) {
          -f $rule_key or do {
             delete $self->configured->{$rule_key};
-            $self->custom->handler->need_save = true;
             false
          }
       } else {
          my ($rulefile, $ext) = split /\@/, $rule_key;
          if ($ext
-             ? ($ext = $Extension::registered_by_URI{$ext},
+             ? (($ext = $Extension::registered_by_URI{$ext}) &&
                 -f $ext->app_dir($self) . "/rules/$rulefile")
              : -f $self->top . "/rules/$rulefile") {
             !$ext || $ext->is_active;
          } else {
             delete $self->configured->{$rule_key};
-            $self->custom->handler->need_save = true;
             false
          }
       }
@@ -351,26 +346,23 @@ sub include_rule {
    if (my $code = $self->rule_code->{$rule_key}) {
       my $credit = $self->credits_by_rulefile->{$rule_key};
       if (defined($credit) && !is_object($credit)) {
-         local bless $self;
          my $reread = new Reconf($self, true);
-         local unshift @INC, $self;
          no strict 'refs';
          local ref *{$self->pkg."::self"} = sub { $reread };
          &$code;
       }
    } else {
-      my ($rulefile, $ext_URI)=split /\@/, $rule_key;
-      local $extension=$Extension::registered_by_URI{$ext_URI} if $ext_URI;
-      my $conf=$self->configured->{$rule_key};
-      if ($conf>0 || !defined($conf)) {
-         local $Shell=new NoShell();
-         include_rules($self, $rulefile);
+      local bless $self;
+      my ($rulefile, $ext_URI) = split /\@/, $rule_key;
+      local $Extension::loading = $Extension::registered_by_URI{$ext_URI} if $ext_URI;
+      my $conf = $self->configured->{$rule_key};
+      local if ($conf > 0 || !defined($conf)) {
+         local $Shell = new NoShell();
       } else {
          delete local $self->rulefiles->{$rule_key};
          delete local $self->configured->{$rule_key};
-         local bless $self;
-         include_rules($self, $rulefile);
       }
+      include_rules($self, $rulefile);
    }
 }
 ###############################################################################################
@@ -405,23 +397,18 @@ sub add_credit {
    $credit
 }
 
-sub add_custom {
-   my $self=shift;
-   if (!$self->simulate_failure  and  my $var=$self->application->custom->re_tie(@_)) {
-      $var->extension=$extension;
-   }
-}
+sub add_custom { }
 
 sub close_preamble {
-   my ($app, $rule_key, $line, $success)=@_;
+   my ($app, $rule_key, $line, $success) = @_;
    if ($line >= $app->preamble_end->{$rule_key}) {
-      ($app->rulefiles->{$rule_key} &&= return 0)=$success;   # avoid repeated execution of the rulefile body
+      ($app->rulefiles->{$rule_key} &&= return 0) = $success;   # avoid repeated execution of the rulefile body
    }
    $success;
 }
 
 sub configure {
-   my ($self, $code, $rule_key, $line, $optional)=@_;
+   my ($self, $code, $rule_key, $line, $optional) = @_;
    if ($self->simulate_failure) {
       $line < $self->preamble_end->{$rule_key};
    } else {
@@ -430,7 +417,7 @@ sub configure {
 }
 
 sub include_required {
-   my ($self, $block, $rule_key, $line)=@_;
+   my ($self, $block, $rule_key, $line) = @_;
    if ($self->simulate_failure) {
       $line < $self->preamble_end->{$rule_key};
    } else {
@@ -439,7 +426,7 @@ sub include_required {
 }
 
 sub require_ext {
-   my ($self, $block, $rule_key, $line)=@_;
+   my ($self, $block, $rule_key, $line) = @_;
    if ($self->simulate_failure) {
       $line < $self->preamble_end->{$rule_key};
    } else {
@@ -448,8 +435,8 @@ sub require_ext {
 }
 
 sub AUTOLOAD {
-   my $self=shift;
-   my ($method)= $AUTOLOAD =~ /::(\w+)$/;
+   my $self = shift;
+   my ($method) = $AUTOLOAD =~ /::(\w+)$/;
    $self->application->$method(@_);
 }
 
@@ -478,22 +465,25 @@ sub add_credit {
 }
 
 sub add_custom {
-   my ($self, $varname, $help_text, $state, $pkg)=@_;
-   $self->application->custom->unset($varname, $pkg) if $state;
+   my ($self, $pkg, $varname, $ref) = @_;
+   if (defined(my $item = UserSettings::get_item($ref))) {
+      $item->reset_value;
+   }
 }
 
 # pretend the last configuration step in the preamble to fail
 sub configure {
-   my ($self, $code, $rule_key, $line)=@_;
+   my ($self, $code, $rule_key, $line) = @_;
    $line < $self->application->preamble_end->{$rule_key} or do {
-      $self->application->configured->{$rule_key}=-$load_time;
-      $self->application->custom->handler->need_save=1;
+      $self->application->configured->{$rule_key} = -$load_time;
       0
    }
 }
 
-*include_required=\&configure;
-*require_ext=\&configure;
+*include_required = \&configure;
+*require_ext = \&configure;
+
+sub include_rule_block {}
 
 ###############################################################################################
 package Polymake::Core::Extension;
@@ -517,11 +507,12 @@ data file, consider storing the outdated URI in the REPLACE section of that
 extension.
 
 If you know that this URI belongs to an obsolete extension which has been
-integrated into the core system, please enter $ignore.
+integrated into the core system, please enter $ignore.  Then the data will be
+processed as if there were no extension attributes in the file.
 
 Finally, if you don't know where to get the extension or don't want to bother
 importing it, please enter $stop to stop loading the data file
-or $skip to load the data without components defined in this extension.
+or $skip to temporarily move the data items into "undecoded" attachments.
 .
    my $dir=$Shell->enter_filename("",
      { prompt => "extension dir",
@@ -545,32 +536,26 @@ or $skip to load the data without components defined in this extension.
 }
 ######################################################################################
 sub obliterate {
-   my ($self)=@_;
+   my ($self) = @_;
    my @drop_apps;
    if ($self->is_active) {
       foreach my $app (list_loaded Application) {
          if ($app->installTop eq $self->dir) {
             $Prefs->obliterate_application($app->prefs);
-            $Custom->obliterate_application($app->custom);
             $app->cpp->obliterate_extension($self, 1);
             $app->delete($app->name);
             push @drop_apps, $app;
          } elsif (delete_from_list($app->extensions, $self)) {
-            $app->custom->obliterate_extension($self);
             $app->cpp->obliterate_extension($self);
-            my $had_configured;
-            while (my ($rule_key)=each %{$app->configured}) {
+            while (my ($rule_key) = each %{$app->configured}) {
                if (index($rule_key, '@'.$self->URI)==0) {
                   delete $app->configured->{$rule_key};
-                  $had_configured=1;
                }
             }
-            if ($had_configured) {
-               $app->custom->set('%configured');
-            }
          }
       }
-      $Prefs->custom->obliterate_extension($self);
+      $Settings->obliterate_extension($self);
+      $Prefs->obliterate_extension($self);
       delete_from_list(\@active, $self);
    }
    delete $registered_by_dir{$self->dir};
@@ -778,14 +763,13 @@ sub add {
       $conflicts{$_}=$self for @{$self->conflicts};
       @{$self->requires}=uniq(@prereqs) if @prereqs;
       push @User::extensions, $ext_dir;
-      $Prefs->custom->set('@extensions');
 
       if ($disable || !$self->configure(@config_options)) {
 	 warn_print( "Extension ", $self->URI, " is disabled on the current architecture $Arch due to problems depicted in the log above.\n",
 		     "In order to activate it, investigate the reasons, install third-party software if instructed to do so,\n",
 		     "then execute a polymake shell command:\n",
 		     "  reconfigure_extension(\"$ext_dir\", options...);" );
-	 $Prefs->custom->set('%disabled_extensions', $ext_dir, 1);
+	 $disabled{$ext_dir} = true;
          undef
       } elsif ($postpone) {
          undef
@@ -830,44 +814,41 @@ sub reconfigure {
    if ($err ne "silent\n") {
       if ($err) {
          print "Configuration script failed with following error(s):\n$err\n";
-         if ($User::disabled_extensions{$self->dir}) {
+         if ($disabled{$self->dir}) {
             print "Extension ", $self->dir,
                   "\nstays disabled until you successfully retry to configure it.\n";
          } else {
             print "Extension ", $self->dir,
                   "\nwill be disabled from the next polymake session on\n",
                   "unless you successfully retry the configuration now\n";
-            $Prefs->custom->set('%disabled_extensions', $self->dir, 1);
+            $disabled{$self->dir} = true;
          }
       } else {
          print "Configuration succeeded without errors.\n";
-         if (delete $User::disabled_extensions{$self->dir}) {
+         if (delete $disabled{$self->dir}) {
             print "Extension ", $self->URI, " is now enabled for current architecture $Arch.\n";
             activate($self);
-            if (defined (my $to_wake=delete $rules_to_wake{$self})) {
+            if (defined(my $to_wake = delete $rules_to_wake{$self})) {
                Application::do_reconfigure(@$to_wake);
             }
 
             # try to enable dependent extensions
             foreach my $ext_dir (@User::extensions) {
-               if ($User::disabled_extensions{$ext_dir} && defined (my $ext=$registered_by_dir{$ext_dir})) {
+               if ($disabled{$ext_dir} && defined(my $ext = $registered_by_dir{$ext_dir})) {
                   unless (grep { !$_->is_active } @{$ext->requires}) {
                      if (-f "$ext_dir/support/configure.pl" || -f "$ext_dir/configure.pl") {
                         print "Now you can try to reconfigure the dependent extension ", $ext->URI, " as well.\n";
                      } elsif ($ext->configure) {
-                        delete $User::disabled_extensions{$ext_dir};
+                        delete $disabled{$ext_dir};
                         print "Extension ", $ext->URI, " is now enabled for current architecture $Arch.\n";
                         activate($ext);
-                        if (defined (my $to_wake=delete $rules_to_wake{$ext})) {
+                        if (defined(my $to_wake = delete $rules_to_wake{$ext})) {
                            Application::do_reconfigure(@$to_wake);
                         }
                      }
                   }
                }
             }
-
-            $Prefs->custom->set('%disabled_extensions');
-
          } else {
             print <<".";
 
@@ -888,7 +869,7 @@ sub write_initial_description {
    print $descr <<'.';
 # This is a description file of a polymake extension rooted in this directory.
 # Please refer to the documentation under
-#   http://www.polymake.org/doku.php/reference/extensions
+#   https://polymake.org/doku.php/user_guide/extend/extensions
 # for detailed explanations of its contents.
 
 # URI is a unique identifier assigned to this extension.
@@ -983,13 +964,22 @@ package Polymake::User;
 ###############################################################################################
 sub help {
    my ($subject) = @_;
-   my $help_delim = $help_delimit ? "-------------------\n" : '';
-   my $app = $application;
-   if ($subject =~ /^($id_re)::/o && $app->used->{$1}) {
-      $subject = $';
-      $app = $app->used->{$1};
+   if (length($subject) == 0) {
+      print $Core::Help::core->display_text;
+      return;
    }
-   if (my @topics = uniq( $app->help->get_topics($subject || "top") )) {
+   my $help_delim = $help_delimit ? "-------------------\n" : '';
+   my $top_help;
+   if ($subject =~ m{^($id_re)(?:$|::|/)}o && defined(my $app = lookup Core::Application($1))) {
+      $subject = $';
+      $top_help = $app->help;
+   } elsif ($subject =~ m{^core/}) {
+      $subject = $';
+      $top_help = $Core::Help::core;
+   } else {
+      $top_help = $application->help;
+   }
+   if (my @topics = uniq( $top_help->get_topics($subject) )) {
       my (@subcats, @subtopics, $need_delim);
       foreach my $topic (@topics) {
          my $text = $topic->display_text;
@@ -1000,7 +990,7 @@ sub help {
          foreach (@{$topic->toc}) {
             if ($topic->topics->{$_}->category & 1) {
                push @subcats, $_;
-            } else {
+            } elsif ($_ ne "any") {
                push @subtopics, $_;
             }
          }
@@ -1008,17 +998,16 @@ sub help {
       if (@subcats || @subtopics) {
          print $help_delim if $need_delim;
       }
-      my $fp = substr($topics[0]->full_path,1);
-      $fp &&= " of $fp";
+      my $fp = $topics[0]->full_path =~ s{^/}{}r || $topics[0]->name;
       if (@subcats) {
-         print "Categories$fp:\n", (join ", ", sort @subcats), "\n";
+         print "Categories of $fp:\n", (join ", ", sort @subcats), "\n";
       }
       if (@subtopics) {
-         print "Subtopics$fp:\n", (join ", ", sort @subtopics), "\n";
+         print "Subtopics of $fp:\n", (join ", ", sort @subtopics), "\n";
       }
-   } elsif (length($subject) && index($subject, "/") < 0
-            and
-            @topics = $app->help->find($subject)) {
+   } elsif (index($subject, "/") < 0
+              and
+            @topics = $top_help->find($subject)) {
 
       if (@topics == 1) {
          print substr($topics[0]->full_path,1), ":\n", $topics[0]->display_text;
@@ -1121,9 +1110,9 @@ sub show_extensions {
                   ($_->credit && do { my $text=$_->credit->text; Core::InteractiveCommands::clean_text($text); $text."\n\n" })
             } @Core::Extension::active[$Core::Extension::num_bundled .. $#Core::Extension::active];
    }
-   if (keys %disabled_extensions) {
+   if (keys %Core::Extension::disabled) {
       print "Following extensions are registered but not configured for current architecture:\n",
-            (map { "  $_\n" } sort keys %disabled_extensions), "\n";
+            (map { "  $_\n" } sort keys %Core::Extension::disabled), "\n";
    }
    if ($Core::Extension::num_bundled) {
       print "Following bundled extensions are configured and used:\n  ",
@@ -1133,7 +1122,7 @@ sub show_extensions {
    if (!@Core::Extension::active) {
       print "No extensions are currently used\n\n";
    }
-   if (keys %disabled_extensions) {
+   if (keys %Core::Extension::disabled) {
       print <<'.';
 To activate an unconfigured extension:  help "reconfigure_extension";
 .
@@ -1156,7 +1145,6 @@ sub found_extension {
       $ext_dir = Cwd::abs_path($ext_dir);
       new_standalone Core::Extension($ext_dir);
       push @extensions, $ext_dir;
-      $Core::Prefs->custom->set('@extensions');
    }
 }
 ###############################################################################################
@@ -1243,22 +1231,21 @@ sub obliterate_extension {
    my $what = shift;
    my $ext = lookup Core::Extension($what);
    $ext->may_obliterate;
-   $Core::Prefs->custom->delete_from_private_list('@extensions', string_list_index(\@extensions, $ext->dir))
-     or die "Extension ", $ext->URI, " can't be obliterated: it is included by global configuration\n";
-
+   my $index = string_list_index(\@extensions, $ext->dir);
+   if ($index < Core::UserSettings::get_item(\@extensions)->reset_to) {
+      die "Extension ", $ext->URI, " can't be obliterated: it is included by global configuration\n";
+   }
+   splice @extensions, $index, 1;
    foreach my $app ($ext->obliterate) {
-      if ($app==$default_application) {
-         $Core::Prefs->custom->reset('$default_application');
+      if ($app == $default_application) {
+         reset_custom $default_application;
       }
-      if ($app==$application) {
+      if ($app == $application) {
          application($default_application);
       }
       delete_string_from_list(\@start_applications, $app->name);
    }
-   $Core::Prefs->custom->set('@start_applications');
-   if (delete $disabled_extensions{$ext->dir}) {
-      $Core::Prefs->custom->set('%disabled_extensions');
-   }
+   delete $Core::Extension::disabled{$ext->dir};
 }
 ###############################################################################################
 sub reconfigure {
@@ -1273,15 +1260,9 @@ sub unconfigure {
 ###############################################################################################
 sub show_unconfigured {
    my (%shown_credits, $credit, $shown);
-   foreach my $rule_key ($application->list_configured(-1)) {
-      print "$rule_key\n",
-            ($credit = $application->first_credit_in($rule_key) and !($shown_credits{$credit->product}++))
-            ? ($credit->text, "\n") : ();
-      $shown = true;
-   }
-   while (my ($name, $app)=each %{$application->used}) {
+   foreach my $app ($application, values %{$application->used}) {
       foreach my $rule_key ($app->list_configured(-1)) {
-         print "$name\::$rule_key\n",
+         print $app != $application ? $app->name."::" : "", $rule_key, "\n",
                ($credit = $app->first_credit_in($rule_key) and !($shown_credits{$credit->product}++))
                ? ($credit->text, "\n") : ();
          $shown = true;
@@ -1294,9 +1275,15 @@ sub show_unconfigured {
 ###############################################################################################
 sub export_configured {
    my $filename = shift;
-   my $opts = @_==1 && ref($_[0]) eq "HASH" ? shift : { @_ };
+   my $opts = @_==1 && is_hash($_[0]) ? shift : { @_ };
    replace_special_paths($filename);
-   $Core::Custom->export($filename, $opts, $Core::Prefs->custom);
+
+   my @opts = delete @$opts{qw(include_imported suppress)};
+   if (keys %$opts) {
+      die "unknown option", keys(%$opts)>1 && "s", ": ", join(" ", keys(%$opts)), "\n";
+   }
+
+   $Settings->save_excerpt($filename, @opts);
 }
 
 1
